#include "frame.h"

frame::frame(dev *devo, Window w, int frametype) {
XWindowAttributes xwa;
  d=devo;
  win=w;
  disp=d->getdisp();

  XGetWindowAttributes(disp, win, &xwa);
  geonorm.x=xwa.x;
  geonorm.y=xwa.y;
  geonorm.h=xwa.height;
  geonorm.w=xwa.width;

  dragging=resizing=shaped=shaded=maximized=minimized=False;
  unmap=1;

  XFetchName(disp, win, &name);

  create_frame();
  link();
}

frame::~frame() {
  XFree(name);
  XReparentWindow(disp, win, *d->getroot(), geonorm.x, geonorm.y);

  XDestroyWindow(disp, frm);
  XRemoveFromSaveSet(disp, win);

  XSync(disp, 0);

  if(d->head_frame==this) d->head_frame=next;
  if(last) last->next=next;
  if(next) next->last=last;
}


void frame::create_frame() {
XSetWindowAttributes xwa;
  xwa.background_pixel=BlackPixel(disp, d->getscreen());
  xwa.override_redirect=True;
  xwa.event_mask=ButtonPressMask|ButtonReleaseMask|KeyPressMask|SubstructureRedirectMask|SubstructureNotifyMask|EnterWindowMask;

  frm=XCreateWindow(disp, *d->getroot(), geonorm.x-1, geonorm.y-2, geonorm.w+2, geonorm.h+2, 0, CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect|CWBackPixel|CWEventMask, &xwa);
  XReparentWindow(disp, win, frm, 1, 1);
  XMapWindow(disp, frm);
  XMapWindow(disp, win); // if its a unmapped window
  XSetWindowBorderWidth(disp, win, 0);
  XAddToSaveSet(disp, win);

  XSelectInput(disp, win, PropertyChangeMask|EnterWindowMask);
}

void frame::link() {
  next=last=None;

  if(d->head_frame) {
    d->head_frame->last=this;
    next=d->head_frame;
  }
  d->head_frame=this;

}

void frame::setwin() {
 XConfigureEvent xcev={ConfigureNotify, 0, True, disp, win, win, geonorm.x, geonorm.y, geonorm.w, geonorm.h, 0, 0, 0};
 XSendEvent(disp, win, False, StructureNotifyMask, (XEvent*)&xcev);
}


void frame::drag(XEvent *ev) {
if(maximized) return;

  switch (ev->type) {
  case ButtonPress:
    geodrag.x=geonorm.x;
    geodrag.y=geonorm.y;
    d->mouse(&geodrag.w, &geodrag.h);
    dragging=True;
    break;
  case MotionNotify:
    geonorm.x=geodrag.x+((*ev).xmotion.x_root - geodrag.w);
    geonorm.y=geodrag.y+((*ev).xmotion.y_root - geodrag.h);
    XMoveWindow(disp, frm, geonorm.x-1, geonorm.y-1);
    break;
  case ButtonRelease:
    dragging=False;
    break;
  }

  setwin();
}


void frame::resize(XEvent *ev) {
if(maximized) return;

  switch (ev->type) {
  case ButtonPress:
    geosize.w=geonorm.w;
    geosize.h=geonorm.h;
    d->mouse(&geosize.x, &geosize.y);
    resizing=True;
    break;
  case MotionNotify:
    geonorm.w=geosize.w+((*ev).xmotion.x_root - geosize.x);
    geonorm.h=geosize.h+((*ev).xmotion.y_root - geosize.y);
    XResizeWindow(disp, frm, geonorm.w+2, geonorm.h+2);
    XResizeWindow(disp, win, geonorm.w, geonorm.h);
    break;
  case ButtonRelease:
    resizing=False;
    break;
  }

  setwin();
}

void frame::kill(bool destroy) {
XEvent ev;
Atom del=XInternAtom(disp, "WM_DELETE_WINDOW", False), 
     pro=XInternAtom(disp, "WM_PROTOCOLS", False);
 
  if(destroy) { XKillClient(disp, win); return; }

  ev.xclient.window=win;
  ev.type=ClientMessage;
  ev.xclient.format=32;
  ev.xclient.message_type=pro;
  ev.xclient.data.l[0]=(long)del;
  ev.xclient.data.l[1]=CurrentTime;
  XSendEvent(disp, win, False, NoEventMask, &ev);
}




